@@ -18,6 +18,7 @@
 #include "language.h"
 #include "doxygen.h"
 #include "outputgen.h"
+#include "outputlist.h"
 #include "dot.h"
 #include "message.h"
 #include "config.h"
@@ -36,6 +37,8 @@
 #include "fileinfo.h"
 #include "indexlist.h"
 #include "growbuf.h"
+#include "portable.h"
+#include "codefragment.h"
 
 static const int NUM_HTML_LIST_TYPES = 4;
 static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
@@ -118,7 +121,11 @@ static bool mustBeOutsideParagraph(const DocNodeVariant &n)
                                  /* <hr> */         DocHorRuler,
                                  /* <blockquote> */ DocHtmlBlockQuote,
                                  /* \parblock */    DocParBlock,
+                                 /* \dotfile */     DocDotFile,
+                                 /* \mscfile */     DocMscFile,
+                                 /* \diafile */     DocDiaFile,
                                  /* <details> */    DocHtmlDetails,
+                                 /* <summary> */    DocHtmlSummary,
                                                     DocIncOperator >(n))
   {
     return TRUE;
@@ -273,9 +280,9 @@ static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAl
 
 //-------------------------------------------------------------------------
 
-HtmlDocVisitor::HtmlDocVisitor(TextStream &t,CodeOutputInterface &ci,
-                               const Definition *ctx)
-  : m_t(t), m_ci(ci), m_ctx(ctx)
+HtmlDocVisitor::HtmlDocVisitor(TextStream &t,OutputCodeList &ci,
+                               const Definition *ctx,const QCString &fn)
+  : m_t(t), m_ci(ci), m_ctx(ctx), m_fileName(fn)
 {
   if (ctx) m_langExt=ctx->getDefFileExtension();
 }
@@ -336,7 +343,7 @@ void HtmlDocVisitor::operator()(const DocSymbol &s)
   }
   else
   {
-    const char *res = HtmlEntityMapper::instance()->html(s.symbol());
+    const char *res = HtmlEntityMapper::instance().html(s.symbol());
     if (res)
     {
       m_t << res;
@@ -344,7 +351,7 @@ void HtmlDocVisitor::operator()(const DocSymbol &s)
     else
     {
       err("HTML: non supported HTML-entity found: %s\n",
-          HtmlEntityMapper::instance()->html(s.symbol(),TRUE));
+          HtmlEntityMapper::instance().html(s.symbol(),TRUE));
     }
   }
 }
@@ -352,7 +359,7 @@ void HtmlDocVisitor::operator()(const DocSymbol &s)
 void HtmlDocVisitor::operator()(const DocEmoji &s)
 {
   if (m_hide) return;
-  const char *res = EmojiEntityMapper::instance()->unicode(s.index());
+  const char *res = EmojiEntityMapper::instance().unicode(s.index());
   if (res)
   {
     m_t << "<span class=\"emoji\">" << res << "</span>";
@@ -375,11 +382,11 @@ void HtmlDocVisitor::writeObfuscatedMailAddress(const QCString &url)
     if (!url.isEmpty())
     {
       const char *p = url.data();
-      uint size=3;
+      uint32_t size=3;
       while (*p)
       {
         m_t << "+'";
-        for (uint j=0;j<size && *p;j++)
+        for (uint32_t j=0;j<size && *p;j++)
         {
           p = writeUTF8Char(m_t,p);
         }
@@ -407,10 +414,10 @@ void HtmlDocVisitor::operator()(const DocURL &u)
     {
       const char *p = url.data();
       // also obfuscate the address as shown on the web page
-      uint size=5;
+      uint32_t size=5;
       while (*p)
       {
-        for (uint j=0;j<size && *p;j++)
+        for (uint32_t j=0;j<size && *p;j++)
         {
           p = writeUTF8Char(m_t,p);
         }
@@ -525,9 +532,6 @@ void HtmlDocVisitor::operator()(const DocStyleChange &s)
     case DocStyleChange::Span:
       if (s.enable()) m_t << "<span" << htmlAttribsToString(s.attribs()) << ">";  else m_t << "</span>";
       break;
-    case DocStyleChange::Summary:
-      if (s.enable()) m_t << "<summary" << htmlAttribsToString(s.attribs()) << ">";  else m_t << "</summary>";
-      break;
   }
 }
 
@@ -605,7 +609,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
             dotindex++,
             ".dot"
            );
-        std::ofstream file(fileName.str(),std::ofstream::out | std::ofstream::binary);
+        std::ofstream file = Portable::openOutputStream(fileName);
         if (!file.is_open())
         {
           err("Could not open file %s for writing\n",qPrint(fileName));
@@ -637,7 +641,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
             qPrint(Config_getString(HTML_OUTPUT)+"/inline_mscgraph_"),
             mscindex++
             );
-        std::ofstream file(baseName.str()+".msc",std::ofstream::out | std::ofstream::binary);
+        std::ofstream file = Portable::openOutputStream(baseName.str()+".msc");
         if (!file.is_open())
         {
           err("Could not open file %s.msc for writing\n",qPrint(baseName));
@@ -721,14 +725,14 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
          forceEndParagraph(inc);
          m_ci.startCodeFragment("DoxyCode");
          FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
+         auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,
                                            inc.context(),
                                            inc.text(),
                                            langExt,
                                            inc.isExample(),
                                            inc.exampleFile(),
-                                           fd,   // fileDef,
+                                           fd.get(),   // fileDef,
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
@@ -736,7 +740,6 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                            TRUE,  // show line numbers
                                            m_ctx  // search context
                                            );
-         delete fd;
          m_ci.endCodeFragment("DoxyCode");
          forceStartParagraph(inc);
       }
@@ -764,56 +767,19 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
       forceStartParagraph(inc);
       break;
     case DocInclude::Snippet:
-      {
-         forceEndParagraph(inc);
-         m_ci.startCodeFragment("DoxyCode");
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           0,
-                                           -1,    // startLine
-                                           -1,    // endLine
-                                           TRUE,  // inlineFragment
-                                           0,     // memberDef
-                                           FALSE, // show line number
-                                           m_ctx  // search context
-                                          );
-         m_ci.endCodeFragment("DoxyCode");
-         forceStartParagraph(inc);
-      }
-      break;
-    case DocInclude::SnipWithLines:
-      {
-         forceEndParagraph(inc);
-         m_ci.startCodeFragment("DoxyCode");
-         FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,
-                                           lineBlock(inc.text(),inc.blockId()),
-                                           -1,    // endLine
-                                           FALSE, // inlineFragment
-                                           0,     // memberDef
-                                           TRUE,  // show line number
-                                           m_ctx  // search context
-                                          );
-         delete fd;
-         m_ci.endCodeFragment("DoxyCode");
-         forceStartParagraph(inc);
-      }
-      break;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
+    case DocInclude::SnippetTrimLeft:
+    case DocInclude::SnippetWithLines:
+      forceEndParagraph(inc);
+      m_ci.startCodeFragment("DoxyCode");
+      CodeFragmentManager::instance().parseCodeFragment(m_ci,
+                                       inc.file(),
+                                       inc.blockId(),
+                                       inc.context(),
+                                       inc.type()==DocInclude::SnippetWithLines,
+                                       inc.type()==DocInclude::SnippetTrimLeft
+                                      );
+      m_ci.endCodeFragment("DoxyCode");
+      forceStartParagraph(inc);
       break;
   }
 }
@@ -837,7 +803,7 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      FileDef *fd = 0;
+      std::unique_ptr<FileDef> fd;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -850,7 +816,7 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
                                 langExt,
                                 op.isExample(),
                                 op.exampleFile(),
-                                fd,     // fileDef
+                                fd.get(),     // fileDef
                                 op.line(),    // startLine
                                 -1,    // endLine
                                 FALSE, // inline fragment
@@ -858,7 +824,6 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
                                 op.showLineNo(),  // show line numbers
                                 m_ctx  // search context
                                );
-      if (fd) delete fd;
     }
     pushHidden(m_hide);
     m_hide=TRUE;
@@ -1275,7 +1240,8 @@ static bool determineIfNeedsTag(const DocPara &p)
                          DocXRefItem,
                          DocHtmlBlockQuote,
                          DocParBlock,
-                         DocHtmlDetails
+                         DocHtmlDetails,
+                         DocHtmlSummary
                          >(*p.parent()))
     {
       needsTag = TRUE;
@@ -1334,7 +1300,7 @@ void HtmlDocVisitor::operator()(const DocPara &p)
     if (strlen(contexts[t]))
       m_t << "<p class=\"" << contexts[t] << "\"" << htmlAttribsToString(p.attribs()) << ">";
     else
-      m_t << "<p " << htmlAttribsToString(p.attribs()) << ">";
+      m_t << "<p" << htmlAttribsToString(p.attribs()) << ">";
   }
 
   visitChildren(p);
@@ -1474,7 +1440,10 @@ void HtmlDocVisitor::operator()(const DocSection &s)
   m_t << "<h" << s.level() << ">";
   m_t << "<a class=\"anchor\" id=\"" << s.anchor();
   m_t << "\"></a>\n";
-  filter(convertCharEntitiesToUTF8(s.title()));
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "</h" << s.level() << ">\n";
   visitChildren(s);
   forceStartParagraph(s);
@@ -1628,11 +1597,24 @@ void HtmlDocVisitor::operator()(const DocHRef &href)
   m_t << "</a>";
 }
 
+void HtmlDocVisitor::operator()(const DocHtmlSummary &s)
+{
+  if (m_hide) return;
+  m_t << "<summary " << htmlAttribsToString(s.attribs()) << ">\n";
+  visitChildren(s);
+  m_t << "</summary>\n";
+}
+
 void HtmlDocVisitor::operator()(const DocHtmlDetails &d)
 {
   if (m_hide) return;
   forceEndParagraph(d);
   m_t << "<details " << htmlAttribsToString(d.attribs()) << ">\n";
+  auto summary = d.summary();
+  if (summary)
+  {
+    std::visit(*this,*summary);
+  }
   visitChildren(d);
   m_t << "</details>\n";
   forceStartParagraph(d);
@@ -1770,6 +1752,7 @@ void HtmlDocVisitor::operator()(const DocDotFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"dotgraph\">\n";
   writeDotFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1782,12 +1765,14 @@ void HtmlDocVisitor::operator()(const DocDotFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocMscFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"mscgraph\">\n";
   writeMscFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1800,12 +1785,14 @@ void HtmlDocVisitor::operator()(const DocMscFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocDiaFile &df)
 {
   if (m_hide) return;
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(HTML_OUTPUT)+"/"+stripPath(df.file()));
+  forceEndParagraph(df);
   m_t << "<div class=\"diagraph\">\n";
   writeDiaFile(df.file(),df.relPath(),df.context(),df.srcFile(),df.srcLine());
   if (df.hasCaption())
@@ -1818,6 +1805,7 @@ void HtmlDocVisitor::operator()(const DocDiaFile &df)
     m_t << "</div>\n";
   }
   m_t << "</div>\n";
+  forceStartParagraph(df);
 }
 
 void HtmlDocVisitor::operator()(const DocLink &lnk)
@@ -1976,8 +1964,10 @@ void HtmlDocVisitor::operator()(const DocXRefItem &x)
   bool anonymousEnum = x.file()=="@";
   if (!anonymousEnum)
   {
+    QCString fn = x.file();
+    addHtmlExtensionIfMissing(fn);
     m_t << "<dl class=\"" << x.key() << "\"><dt><b><a class=\"el\" href=\""
-        << x.relPath() << addHtmlExtensionIfMissing(x.file())
+        << x.relPath() << fn
         << "#" << x.anchor() << "\">";
   }
   else
@@ -1985,7 +1975,6 @@ void HtmlDocVisitor::operator()(const DocXRefItem &x)
     m_t << "<dl class=\"" << x.key() << "\"><dt><b>";
   }
   filter(x.title());
-  m_t << ":";
   if (!anonymousEnum) m_t << "</a>";
   m_t << "</b></dt><dd>";
   visitChildren(x);
@@ -2026,7 +2015,7 @@ void HtmlDocVisitor::operator()(const DocVhdlFlow &vf)
     forceEndParagraph(vf);
     QCString fname=FlowChart::convertNameToFileName();
     m_t << "<p>";
-    m_t << "flowchart: " ; // TODO: translate me
+    m_t << theTranslator->trFlowchart();
     m_t << "<a href=\"";
     m_t << fname;
     m_t << ".svg\">";
@@ -2072,7 +2061,7 @@ void HtmlDocVisitor::filter(const QCString &str, const bool retainNewline)
         break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           if (uc<32 && !isspace(c)) // non-printable control characters
           {
             m_t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
@@ -2117,7 +2106,7 @@ QCString HtmlDocVisitor::filterQuotedCdataAttr(const QCString &str)
         break;
       default:
         {
-          uchar uc = static_cast<uchar>(c);
+          uint8_t uc = static_cast<uint8_t>(c);
           if (uc<32 && !isspace(c)) // non-printable control characters
           {
             growBuf.addStr("&#x24");
@@ -2152,12 +2141,12 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,
     m_t << "<a class=\"el\" ";
   }
   m_t << "href=\"";
-  m_t << externalRef(relPath,ref,TRUE);
-  if (!file.isEmpty())
-  {
-    m_t << addHtmlExtensionIfMissing(file);
-  }
-  if (!anchor.isEmpty()) m_t << "#" << anchor;
+  QCString fn = file;
+  addHtmlExtensionIfMissing(fn);
+  m_t << createHtmlUrl(relPath,ref,true,
+                       m_fileName == Config_getString(HTML_OUTPUT)+"/"+fn,
+                       fn,
+                       anchor);
   m_t << "\"";
   if (!tooltip.isEmpty()) m_t << " title=\"" << convertToHtml(tooltip) << "\"";
   m_t << ">";
@@ -2231,7 +2220,7 @@ void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relP
 }
 
 void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName, const QCString &relPath,
-                                       const QCString &,const QCString &srcFile,int srcLine)
+                                       const QCString &,const QCString &/* srcFile */,int /* srcLine */)
 {
   QCString baseName=fileName;
   int i;
