@@ -1,8 +1,6 @@
 /******************************************************************************
  *
- *
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2023 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -48,13 +46,15 @@ class LatexCodeGenerator
                       const SourceLinkInfo &
                      )  {}
     void writeLineNumber(const QCString &,const QCString &,const QCString &,int,bool);
-    void startCodeLine(bool);
+    void startCodeLine(int);
     void endCodeLine();
     void startFontClass(const QCString &);
     void endFontClass();
     void writeCodeAnchor(const QCString &) {}
     void startCodeFragment(const QCString &style);
     void endCodeFragment(const QCString &style);
+    void startFold(int,const QCString &,const QCString &) {}
+    void endFold() {}
 
     // extra methods not part of CodeOutputInterface
     void incUsedTableLevel() { m_usedTableLevel++; }
@@ -224,7 +224,7 @@ class LatexGenerator : public OutputGenerator
     void writeSplitBar(const QCString &) {}
     void writeNavigationPath(const QCString &) {}
     void writeLogo() {}
-    void writeQuickLinks(bool,HighlightedItem,const QCString &) {}
+    void writeQuickLinks(HighlightedItem,const QCString &,bool) {}
     void writeSummaryLink(const QCString &,const QCString &,const QCString &,bool) {}
     void startContents() {}
     void endContents() {}
@@ -263,6 +263,8 @@ class LatexGenerator : public OutputGenerator
     void endParameterType();
     void startParameterName(bool);
     void endParameterName(bool,bool,bool);
+    void startParameterDefVal(const char *s) { docify(s); startTypewriter(); }
+    void endParameterDefVal() { endTypewriter(); }
     void startParameterList(bool);
     void endParameterList();
     void exceptionEntry(const QCString &,bool);
