@@ -1,9 +1,6 @@
 /******************************************************************************
  *
- *
- *
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2022 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -27,6 +24,7 @@
 #include "msc.h"
 #include "util.h"
 #include "rtfstyle.h"
+#include "rtfgen.h"
 #include "message.h"
 #include "parserintf.h"
 #include "msc.h"
@@ -37,6 +35,8 @@
 #include "emoji.h"
 #include "plantuml.h"
 #include "fileinfo.h"
+#include "portable.h"
+#include "codefragment.h"
 
 //#define DBG_RTF(x) m_t << x
 #define DBG_RTF(x) do {} while(0)
@@ -55,9 +55,9 @@ static QCString align(const DocHtmlCell &cell)
   return "";
 }
 
-RTFDocVisitor::RTFDocVisitor(TextStream &t,CodeOutputInterface &ci,
-                             const QCString &langExt)
-  : m_t(t), m_ci(ci), m_langExt(langExt)
+RTFDocVisitor::RTFDocVisitor(TextStream &t,OutputCodeList &ci,
+                             const QCString &langExt, int hierarchyLevel)
+  : m_t(t), m_ci(ci), m_langExt(langExt), m_hierarchyLevel(hierarchyLevel)
 {
 }
 
@@ -128,14 +128,14 @@ void RTFDocVisitor::operator()(const DocSymbol &s)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visit(DocSymbol)}\n");
-  const char *res = HtmlEntityMapper::instance()->rtf(s.symbol());
+  const char *res = HtmlEntityMapper::instance().rtf(s.symbol());
   if (res)
   {
     m_t << res;
   }
   else
   {
-    err("RTF: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance()->html(s.symbol(),TRUE));
+    err("RTF: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance().html(s.symbol(),TRUE));
   }
   m_lastIsPara=FALSE;
 }
@@ -144,7 +144,7 @@ void RTFDocVisitor::operator()(const DocEmoji &s)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::visit(DocEmoji)}\n");
-  const char *res = EmojiEntityMapper::instance()->unicode(s.index());
+  const char *res = EmojiEntityMapper::instance().unicode(s.index());
   if (res)
   {
     const char *p = res;
@@ -286,9 +286,6 @@ void RTFDocVisitor::operator()(const DocStyleChange &s)
       break;
     case DocStyleChange::Div:  /* HTML only */ break;
     case DocStyleChange::Span: /* HTML only */ break;
-    case DocStyleChange::Summary: /* emulation of the <summary> tag inside a <details> tag */
-      if (s.enable()) m_t << "{\\b ";      else m_t << "}\\par ";
-      break;
   }
 }
 
@@ -351,7 +348,7 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
             dotindex++,
             ".dot"
            );
-        std::ofstream file(fileName.str(),std::ofstream::out | std::ofstream::binary);
+        std::ofstream file = Portable::openOutputStream(fileName);
         if (!file.is_open())
         {
           err("Could not open file %s for writing\n",qPrint(fileName));
@@ -380,7 +377,7 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
             mscindex++,
             ".msc"
            );
-        std::ofstream file(baseName.str(),std::ofstream::out | std::ofstream::binary);
+        std::ofstream file = Portable::openOutputStream(baseName);
         if (!file.is_open())
         {
           err("Could not open file %s for writing\n",qPrint(baseName));
@@ -449,20 +446,19 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
          m_t << "\\par\n";
          m_t << rtf_Style_Reset << getStyle("CodeExample");
          FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
+         auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
                                            inc.text(),
                                            langExt,
                                            inc.isExample(),
                                            inc.exampleFile(),
-                                           fd,   // fileDef,
+                                           fd.get(),   // fileDef,
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
                                            0,     // memberDef
                                            TRUE   // show line numbers
 					   );
-         delete fd;
          m_t << "\\par";
          m_t << "}\n";
       }
@@ -504,47 +500,20 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
       m_t << "}\n";
       break;
     case DocInclude::Snippet:
+    case DocInclude::SnippetTrimLeft:
+    case DocInclude::SnippetWithLines:
       m_t << "{\n";
       if (!m_lastIsPara) m_t << "\\par\n";
       m_t << rtf_Style_Reset << getStyle("CodeExample");
-      getCodeParser(inc.extension()).parseCode(m_ci,
-                                        inc.context(),
-                                        extractBlock(inc.text(),inc.blockId()),
-                                        langExt,
-                                        inc.isExample(),
-                                        inc.exampleFile()
-                                       );
+      CodeFragmentManager::instance().parseCodeFragment(m_ci,
+                                         inc.file(),
+                                         inc.blockId(),
+                                         inc.context(),
+                                         inc.type()==DocInclude::SnippetWithLines,
+                                         inc.type()==DocInclude::SnippetTrimLeft
+                                        );
       m_t << "}";
       break;
-    case DocInclude::SnipWithLines:
-      {
-         FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
-         m_t << "{\n";
-         if (!m_lastIsPara) m_t << "\\par\n";
-         m_t << rtf_Style_Reset << getStyle("CodeExample");
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,
-                                           lineBlock(inc.text(),inc.blockId()),
-                                           -1,    // endLine
-                                           FALSE, // inlineFragment
-                                           0,     // memberDef
-                                           TRUE   // show line number
-                                          );
-         delete fd;
-         m_t << "}";
-      }
-      break;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
-      break;
   }
   m_lastIsPara=TRUE;
 }
@@ -573,7 +542,7 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      FileDef *fd = 0;
+      std::unique_ptr<FileDef> fd = 0;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -582,14 +551,13 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
 
       getCodeParser(locLangExt).parseCode(m_ci,op.context(),op.text(),langExt,
                                         op.isExample(),op.exampleFile(),
-                                        fd,     // fileDef
+                                        fd.get(),     // fileDef
                                         op.line(),    // startLine
                                         -1,    // endLine
                                         FALSE, // inline fragment
                                         0,     // memberDef
                                         op.showLineNo()  // show line numbers
                                        );
-      if (fd) delete fd;
     }
     pushHidden(m_hide);
     m_hide=TRUE;
@@ -862,15 +830,23 @@ void RTFDocVisitor::operator()(const DocSection &s)
   m_t << "{{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = std::min(s.level()+1,4);
+  int level = std::min(s.level()+2+m_hierarchyLevel,4);
+  if (level <= 0)
+    level = 1;
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading.str()].reference() << "\n";
   // make table of contents entry
-  filter(s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "\n\\par" << "}\n";
   m_t << "{\\tc\\tcl" << level << " \\v ";
-  filter(s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "}\n";
   m_lastIsPara=TRUE;
   visitChildren(s);
@@ -1131,25 +1107,34 @@ void RTFDocVisitor::operator()(const DocHRef &href)
   m_lastIsPara=FALSE;
 }
 
+void RTFDocVisitor::operator()(const DocHtmlSummary &s)
+{
+  if (m_hide) return;
+  m_t << "{\\b ";
+  visitChildren(s);
+  m_t << "}\\par ";
+}
+
 void RTFDocVisitor::operator()(const DocHtmlDetails &d)
 {
   if (m_hide) return;
-  //m_lastIsPara=TRUE;
-  //m_t << "{\n";
-  //m_t << "\\par\n";
-  //visitChildren(d);
-  //m_t << "\\par";
-  //m_t << "}\n";
-  //m_lastIsPara=TRUE;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocHtmlDetails &)}\n");
   if (!m_lastIsPara) m_t << "\\par\n";
-  m_t << "{"; // start desc
-  incIndentLevel();
-  m_t << rtf_Style_Reset << getStyle("DescContinue");
+  auto summary = d.summary();
+  if (summary)
+  {
+    std::visit(*this,*summary);
+    m_t << "{"; // start desc
+    incIndentLevel();
+    m_t << rtf_Style_Reset << getStyle("DescContinue");
+  }
   visitChildren(d);
   if (!m_lastIsPara) m_t << "\\par\n";
-  decIndentLevel();
-  m_t << "}"; // end desc
+  if (summary)
+  {
+    decIndentLevel();
+    m_t << "}"; // end desc
+  }
   m_lastIsPara=TRUE;
 }
 
@@ -1160,7 +1145,9 @@ void RTFDocVisitor::operator()(const DocHtmlHeader &header)
   m_t << "{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = std::min(header.level(),5);
+  int level = std::min(header.level()+m_hierarchyLevel,5);
+  if (level <= 0)
+    level = 1;
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading.str()].reference();
